#include <proto.h>

#include "gtseparator.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtSeparator class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init separator.
*
\************************************************************************/
GtSeparator *GtMkSeparator(char *n,GtManager *p,GtTag *tgs)
{
	GtSeparator *obj;
	
	if(!(obj=new GtSeparator()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtSeparator object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init separator.
*
\************************************************************************/
GtSeparator *GtMkSeparator(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtSeparator *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkSeparator(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}
	
/************************************************************************\
*
* Setup empty separator.
*
\************************************************************************/
GtSeparator::GtSeparator(void)
{
	orient=GTO_HORIZ;
	stype=GTF_DBL_IN;
}

/************************************************************************\
*
* Setup separator.
*
\************************************************************************/
sbool GtSeparator::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	
	if(!GtWidget::Init(n,p,tgs))
		return(FALSE);
	app->GetRes(this,GTR_ORIENT,GTRT_ORIENT,&orient);
	app->GetRes(this,GTR_FRAME,GTRT_FRAME,&stype);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Orient:
				orient=t->d.val;
				if(orient>GTO_LAST_NR)
					orient=GTO_HORIZ;
				break;
			case GT_FrameType:
				stype=t->d.val;
				if(stype>GTF_LAST_NR)
					stype=GTF_DBL_IN;
				break;
		}
	}
	switch(stype)
	{
		case GTF_NONE:
		case GTF_SIMPLE:
		case GTF_IN:
		case GTF_OUT:
			this->SetMinMaxSize(1,1,GTS_MAX,GTS_MAX);
			break;
		case GTF_DBL_IN:
		case GTF_DBL_OUT:
			this->SetMinMaxSize(2,2,GTS_MAX,GTS_MAX);
			break;
	}
	return(TRUE);
}

/************************************************************************\
*
* Set separator attributes.
*
\************************************************************************/
sbool GtSeparator::Set(GtTag *tgs)
{
	GtTag *t;
	sbool ch=FALSE;
	
	if(!GtWidget::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Orient:
				orient=t->d.val;
				if(orient>GTO_LAST_NR)
					orient=GTO_HORIZ;
				ch=TRUE;
				break;
			case GT_FrameType:
				stype=t->d.val;
				if(stype>GTF_LAST_NR)
					stype=GTF_DBL_IN;
				ch=TRUE;
				break;
		}
	}
	if(ch)
	{
		switch(stype)
		{
			case GTF_NONE:
			case GTF_SIMPLE:
				this->SetMinMaxSize(1,1,GTS_MAX,GTS_MAX);
				break;
			case GTF_DBL_IN:
			case GTF_DBL_OUT:
			case GTF_IN:
			case GTF_OUT:
				this->SetMinMaxSize(2,2,GTS_MAX,GTS_MAX);
				break;
		}
		this->Refresh();
	}
	return(TRUE);
}

/************************************************************************\
*
* Get separator attributes.
*
\************************************************************************/
void GtSeparator::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtWidget::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Orient:
				*ptr=orient;
				break;
			case GT_FrameType:
				*ptr=stype;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtSeparator::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtSeparator")==0)
		return(TRUE);
	else
		return(GtWidget::IsSubClassOf(cn));
}

/************************************************************************\
*
* Set separator event mask.
*
\************************************************************************/
void GtSeparator::SetEMask(ulong em)
{
	GtWidget::SetEMask(em|ExposureMask);
}

/************************************************************************\
*
* Draw separator.
*
\************************************************************************/
sbool GtSeparator::Draw(XEvent *xe)
{
	int xp,yp;
	
	if(xe && ((xe->type!=Expose) || (xe->xexpose.count!=0)))
		return(TRUE);
	if(flags & GTWF_NOWIN)
	{
		xp=xpos;
		yp=ypos;
	}
	else
	{
		xp=0;
		yp=0;
	}
	if(orient==GTO_HORIZ)
	{
		switch(stype)
		{
			case GTF_SIMPLE:
				gtfore->DrawLine(gtwin,xp,yp+(ysize-1)/2,xsize-1,(ysize-1)/2);
				break;
			case GTF_IN:
			case GTF_DBL_IN:
				gtdrk->DrawLine(gtwin,xp,yp+(ysize-2)/2,xsize-1,(ysize-2)/2);
				gtlht->DrawLine(gtwin,xp,yp+(ysize-2)/2+1,xsize-1,(ysize-2)/2+1);
				break;
			case GTF_OUT:
			case GTF_DBL_OUT:
				gtlht->DrawLine(gtwin,xp,yp+(ysize-2)/2,xsize-1,(ysize-2)/2);
				gtdrk->DrawLine(gtwin,xp,yp+(ysize-2)/2+1,xsize-1,(ysize-2)/2+1);
				break;
		}
	}
	else
	{
		switch(stype)
		{
			case GTF_SIMPLE:
				gtfore->DrawLine(gtwin,xp+(xsize-1)/2,yp,(xsize-1)/2,ysize-1);
				break;
			case GTF_IN:
			case GTF_DBL_IN:
				gtdrk->DrawLine(gtwin,xp+(xsize-2)/2,yp,(xsize-2)/2,ysize-1);
				gtlht->DrawLine(gtwin,xp+(xsize-2)/2+1,yp,(xsize-2)/2+1,ysize-1);
				break;
			case GTF_OUT:
			case GTF_DBL_OUT:
				gtlht->DrawLine(gtwin,xp+(xsize-2)/2,yp,(xsize-2)/2,ysize-1);
				gtdrk->DrawLine(gtwin,xp+(xsize-2)/2+1,yp,(xsize-2)/2+1,ysize-1);
				break;
		}
	}
	return(TRUE);
}
